function [c, ceq] = constraints(coeff1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           Velocity Profile                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a           = [coeff1(1), coeff1(2), coeff1(3), coeff1(4), coeff1(5), coeff1(6), coeff1(7)];
b           = [coeff1(8), coeff1(9), coeff1(10), coeff1(11), coeff1(12), coeff1(13), coeff1(14)];

omega       = 1;
th_dot      = [];
th_ddot     = [];
delta_time  = [];
delta_th    = 0.01;


for th = 0:delta_th:2*pi
   th_tmp1 = 0;
   th_tmp2 = 0;
   for i = 1:length(a)
       j = i - 1;
       th_tmp1 = th_tmp1 + a(i)*sin(j*omega*th) + b(i)*cos(j*omega*th);
       th_tmp2 = th_tmp2 + j*omega*a(i)*cos(j*omega*th) - j*omega*b(i)*sin(j*omega*th);
   end
   th_dot  = [th_dot; [th, th_tmp1]];
   th_ddot = [th_ddot; [th, th_tmp2*th_tmp1]];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                PATH PLANNING                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DH parameters of robot
d1 = .4865;
d4 = .600;
d6 = .065;
a1 = .15;
a2 = .475;
a3 = d4 + d6;

%%Changeable variables 
pos         = [];
pos_joint   = [];
vel_joint   = [];
R1          = .15;
% p0 = ForwardKinematics(q0);
x_c1        = 0.6; %p0(1) - R1;
y_c1        = 0; %p0(2);
z_c1        = 0.13; %p0(3);

for th = 1:length(th_dot)
    %%Third order path planning
    theta       = th_dot(th,1);
    d_theta     = th_dot(th,2);
    dd_theta    = th_ddot(th,2);
    
    %% Position 
    q1          = atan2((R1*sin(theta) + y_c1),(R1*cos(theta) + x_c1));
    l           = sqrt((d1 - z_c1)^2 + (x_c1 - a1 + R1*cos(theta))^2);
    theta_m     = acos((a3^2 + a2^2 - l^2)/(2*a2*a3));
    theta_f     = atan2((d1 - z_c1),(x_c1 - a1 + R1*cos(theta)));
    theta_b     = asin((a3*sin(theta_m))/l) - theta_f;
    q2          = pi/2 - theta_b;
    q3          = pi/2 - theta_m;
    pos         = [pos ForwardKinematics([q1 q2 q3 0 0 0])];
    pos_joint   = [pos_joint; [q1 q2 q3 0 0 0]];
    
    %% Velocity
    d_phi1      = ((R1 * cos(theta) / (R1 * cos(theta) + x_c1) + (R1 * sin(theta) + y_c1) / (R1 * cos(theta) + x_c1) ^ 2 * R1 * sin(theta)) / ((R1 * sin(theta) + y_c1) ^ 2 / (R1 * cos(theta) + x_c1) ^ 2 + 0.1e1));
    d_phi2      = (-0.2e1 * (-0.1e1 / a3 * (0.4e1 - (a2 ^ 2 + a3 ^ 2 - (d1 - z_c1) ^ 2 - (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ 2 / a3 ^ 2 / a2 ^ 2) ^ (-0.1e1 / 0.2e1) * ((d1 - z_c1) ^ 2 + (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ (-0.1e1 / 0.2e1) * (a2 ^ 2 + a3 ^ 2 - (d1 - z_c1) ^ 2 - (x_c1 - a1 + R1 * cos(theta)) ^ 2) / a2 ^ 2 * (x_c1 - a1 + R1 * cos(theta)) * R1 * sin(theta) + a3 * sqrt(0.4e1 - (a2 ^ 2 + a3 ^ 2 - (d1 - z_c1) ^ 2 - (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ 2 / a3 ^ 2 / a2 ^ 2) * ((d1 - z_c1) ^ 2 + (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ (-0.3e1 / 0.2e1) * (x_c1 - a1 + R1 * cos(theta)) * R1 * sin(theta) / 0.2e1) * (-a3 ^ 2 * (0.4e1 - (a2 ^ 2 + a3 ^ 2 - (d1 - z_c1) ^ 2 - (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ 2 / a3 ^ 2 / a2 ^ 2) / ((d1 - z_c1) ^ 2 + (x_c1 - a1 + R1 * cos(theta)) ^ 2) + 0.4e1) ^ (-0.1e1 / 0.2e1) + (d1 - z_c1) / (x_c1 - a1 + R1 * cos(theta)) ^ 2 * R1 * sin(theta) / ((d1 - z_c1) ^ 2 / (x_c1 - a1 + R1 * cos(theta)) ^ 2 + 0.1e1));
    d_phi3      = (0.2e1 * (x_c1 - a1 + R1 * cos(theta)) * R1 * sin(theta) / a3 / a2 * (0.4e1 - (a2 ^ 2 + a3 ^ 2 - (d1 - z_c1) ^ 2 - (x_c1 - a1 + R1 * cos(theta)) ^ 2) ^ 2 / a3 ^ 2 / a2 ^ 2) ^ (-0.1e1 / 0.2e1));
    
    dq1         = d_phi1 * d_theta;
    dq2         = d_phi2 * d_theta;
    dq3         = d_phi3 * d_theta;
    
    vel_joint   = [vel_joint; [dq1 dq2 dq3 0 0 0]];
end

g = 9.81;

q1      = pos_joint(:,1);
q2      = pos_joint(:,2);
q3      = pos_joint(:,3);
q       = [q1 q2 q3]';
dq1     = vel_joint(:,1);
dq2     = vel_joint(:,2);
dq3     = vel_joint(:,3);
dq      = [dq1 dq2 dq3]';

velocity_constraint = [];

for j=1:length(vel_joint(:,1))
    velocity_constraint = [velocity_constraint;
                           abs(dq1(j)) - 1;
                           abs(dq2(j)) - 1;
                           abs(dq3(j)) - 1];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                               Output                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% delta denotes the minimum volocity for \dot \theta along \theta
delta = 0.05;

for i=1:length(th_dot(:,1))
    tmp = th_dot(i,2) - delta;
    th_dot(i,2) = tmp;
end

time_limit = 2*pi/b(1) - 15;

c   = [-1*th_dot(:,2); velocity_constraint; time_limit];
ceq = [];

end