%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          Optimize Coefficients                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all; clear all; clc;

a = 2*[0, 0.001, 0.001, 0, 0, 0, 0];
b = 0.7*[0.5, 0.01, 0.01, 0, 0, 0, 0];

coeff = [a,b]';
first_iter = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                fmincon                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[bestCoeffs, energy] = fmincon(@simOpt, coeff, [], [], [], [], [], [], @constraints);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      Plotting of Optimal Solution                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a           = [bestCoeffs(1), bestCoeffs(2), bestCoeffs(3), bestCoeffs(4), bestCoeffs(5) bestCoeffs(6), bestCoeffs(7)];
b           = [bestCoeffs(8), bestCoeffs(9), bestCoeffs(10), bestCoeffs(11), bestCoeffs(12), bestCoeffs(13), bestCoeffs(14)];

omega       = 1;
th_dot      = [];
th_ddot     = [];
delta_time  = [];
delta_th    = 0.001;


for th = 0:delta_th:2*pi
   th_tmp1 = 0;
   th_tmp2 = 0;
   for i = 1:length(a)
       j = i - 1;
       th_tmp1 = th_tmp1 + a(i)*sin(j*omega*th) + b(i)*cos(j*omega*th);
       th_tmp2 = th_tmp2 + j*omega*a(i)*cos(j*omega*th) - j*omega*b(i)*sin(j*omega*th);
   end
   th_dot  = [th_dot; [th, th_tmp1]];
   th_ddot = [th_ddot; [th, th_tmp2*th_tmp1]];
end

time = [];

for i = 1:length(th_dot(:,1))
    if i == 1
        time = delta_th/abs(th_dot(i,2));
    else
        time = [time, time(i-1) + delta_th/abs(th_dot(i,2))];
    end
end
% Display time extension of optimal trajecory
total_time = time(end)

figure(10)
clf;
plot(time, th_dot(:,2))
xlabel('time [s]')
ylabel('$\dot \theta\;\left[rad/s\right]$','Interpreter','Latex','FontSize',12)
xlim([0 total_time]);
figure(20)
clf;
plot(time, th_ddot(:,2))
xlabel('time [s]')
ylabel('$\ddot \theta\; \left[rad/s^2\right]$','Interpreter','Latex','FontSize',12)
xlim([0 total_time]);
figure(30)
plot(th_dot(:,1),th_dot(:,2))
xlabel('$\theta\;[rad]$','Interpreter','Latex','FontSize',12)
ylabel('$\dot \theta\;\left[rad/s\right]$','Interpreter','Latex','FontSize',12)
xlim([0 2*pi]);
figure(40)
plot(th_ddot(:,1),th_ddot(:,2))
xlabel('$\theta\;[rad]$','Interpreter','Latex','FontSize',12)
ylabel('$\ddot \theta\;\left[rad/s^2\right]$','Interpreter','Latex','FontSize',12)
xlim([0 2*pi]);